@@ -2,28 +2,27 @@ package HTML::Menu::TreeView;
 use strict;
 use warnings;
 require Exporter;
-use vars qw($DefaultClass %EXPORT_TAGS @EXPORT_OK @ISA %anker @TreeView %openArrays);
-$HTML::Menu::TreeView::VERSION = '0.8';
+use vars qw($DefaultClass %EXPORT_TAGS @EXPORT_OK @ISA %anker @TreeView %openArrays @caption $columns $clasic $ffirst $sort $border $orderby $size $style $orderbyColumn $prefix);
+$HTML::Menu::TreeView::VERSION = '0.84';
 @ISA                           = qw(Exporter);
 @HTML::Menu::TreeView::EXPORT_OK =
-  qw(all Tree css jscript setStyle setDocumentRoot getDocumentRoot setSize setClasic clasic preload help folderFirst size style Style documentRoot loadTree saveTree  %anker sortTree orderBy orderByColumn prefix setModern border);
+  qw(border Tree css columns jscript setStyle setDocumentRoot getDocumentRoot setSize setClasic clasic preload help folderFirst size style Style documentRoot loadTree saveTree  %anker sortTree orderBy orderByColumn prefix setModern border);
 %HTML::Menu::TreeView::EXPORT_TAGS = (
-                                      'all'       => [qw(Tree css jscript clasic preload help folderFirst  size documentRoot  loadTree saveTree sortTree orderBy prefix Style orderByColumn  border)],
-                                      'recommend' => [qw(Tree css jscript clasic preload folderFirst size Style documentRoot  loadTree saveTree sortTree orderBy prefix)],
+                                      'all'       => [qw(Tree css jscript clasic columns preload help folderFirst  size documentRoot  loadTree saveTree sortTree orderBy prefix Style orderByColumn  border)],
+                                      'recommend' => [qw(Tree css jscript clasic preload folderFirst help size Style documentRoot  loadTree saveTree sortTree orderBy prefix)],
                                       'standart'  => [qw(Tree css jscript preload size Style documentRoot clasic)],
                                       'backward'  => [qw(setDocumentRoot getDocumentRoot setSize setClasic setStyle style setModern %anker)],
                                       'columns'   => [qw(border columns orderByColumn)],
 );
 $DefaultClass = 'HTML::Menu::TreeView' unless defined $HTML::Menu::TreeView::DefaultClass;
-our $id    = 'a';
-our $style = 'Crystal';
-our $size  = 16;
-our $path  = '%PATH%';
-our ($clasic, $ffirst, $sort, $border, $columns) = (0) x 5;
+our $id       = 'a';
+our $path     = '%PATH%';
 our $saveFile = './TreeViewDump.pl';
-our $orderby  = 'text';
-our $pref     = '';
-our (@caption, $orderbyColumn);
+$style = 'Crystal';
+$size  = 16;
+($clasic, $ffirst, $sort, $border, $columns) = (0) x 5;
+$orderby = 'text';
+$prefix  = '';
 
 %anker = (
           href        => 'URI for linked resource',
@@ -186,9 +185,11 @@ HTML::Menu::TreeView is a Modul to build an Html tree of an AoH.
 
 =head1 Changes
 
-0.8
+0.84
 
-some fixes
+        make usage of Module::Build
+        Overwrought Code.
+        a few fixes
 
 =head1 Public
 
@@ -236,14 +237,14 @@ sub css {
         $self->documentRoot($p[0]) if(defined $p[0]);
         my $fh   = gensym;
         my $file = "$path/style/$style/$size/html-menu-treeview/$style.css";
-        open $fh, "$file" or warn "HTML::Menu::TreeView::css $/ no style '$style.css' $style.css found $/  in  $path/style/html-menu-treeview/$style/ $/ $! $/";
+        open $fh, $file or warn "HTML::Menu::TreeView::css $/ no style '$style.css' $style.css found $/  in  $path/style/html-menu-treeview/$style/ $/ $! $/";
         seek $fh, 0, 0;
         my @lines = <$fh>;
         close $fh;
         my $css;
 
         foreach my $l (@lines) {
-                $l =~ s?/style/?$pref/style/?g;
+                $l =~ s?/style/?$prefix/style/?g;
                 $css .= $l;
         }
         return $css;
@@ -289,7 +290,7 @@ sub jscript {
         $self->documentRoot($p[0]) if(defined $p[0]);
         my $fh   = gensym;
         my $file = "$path/style/treeview.js";
-        open $fh, "$file" or warn "HTML::Menu::TreeView::jscript $/ $! $/";
+        open $fh, $file or warn "HTML::Menu::TreeView::jscript $/ $! $/";
         seek $fh, 0, 0;
         my @js = <$fh>;
         close $fh;
@@ -321,14 +322,14 @@ sub preload {
         $self->documentRoot($p[0]) if(defined $p[0]);
         my $fh   = gensym;
         my $file = "$path/style/$style/$size/html-menu-treeview/preload.js";
-        open $fh, "$file" or warn "HTML::Menu::TreeView::preload $/ $! $/";
+        open $fh, $file or warn "HTML::Menu::TreeView::preload $/ $! $/";
         seek $fh, 0, 0;
         my @lines = <$fh>;
         close $fh;
         my $preload;
 
         foreach my $l (@lines) {
-                $l =~ s?/style/?$pref/style/?g;
+                $l =~ s?/style/?$prefix/style/?g;
                 $preload .= $l;
         }
         return $preload;
@@ -405,14 +406,14 @@ sub Tree {
                 }
                 $r .= "//-->\n</script>";
         }
-        $r .= qq(<table border="0" cellpadding="2" cellspacing="1" summary="Tree"><tr><td><table align="left" border="0" cellpadding="0" cellspacing="0" summary="Tree" width="100%" class="treeview"><colgroup><col width="$size"/></colgroup>);
+        $r .= qq(<table border="0" cellpadding="2" cellspacing="1" summary="Tree"><tr><td><table align="left" border="0" cellpadding="0" cellspacing="0" summary="Tree" width="100%" class="treeview$size"><colgroup><col width="$size"/></colgroup>);
         if(defined @caption) {
                 my $class = $border ? "captionBorder$size" : "caption$size";
                 $r .= qq(<tr><td class="$class"></td><td class="$class">$caption[0]</td></tr>);
         }
         $r .= $self->{tree} . '</table></td>';
         if(defined $self->{subtree}) {
-                $r .= '<td ><table align="left" border="0" cellpadding="0" cellspacing="0" summary="subTree" width="100%" >';
+                $r .= '<td><table align="left" border="0" cellpadding="0" cellspacing="0" summary="subTree" width="100%" >';
                 if(defined @caption) {
                         my $class = $border ? "captionBorder$size" : "caption$size";
                         $r .= '<tr>';
@@ -421,7 +422,7 @@ sub Tree {
                         }
                         $r .= '</tr>';
                 }
-                @caption = undef;
+                undef @caption;
                 $r .= $self->{subtree} . '</table></td>';
 
         }
@@ -610,9 +611,9 @@ return the prefix in void context.
 sub prefix {
         my ($self, @p) = getSelf(@_);
         if(defined $p[0]) {
-                $pref = $p[0];
+                $prefix = $p[0];
         } else {
-                return $pref;
+                return $prefix;
         }
 }
 
@@ -856,6 +857,12 @@ an array of columns
 
 columns => [ 1,2,3,4,5]
 
+=item empty.
+
+set it true if you ant a closed Folder,
+
+which load a location onclick, you must additional set the href attribute.
+
 =back
 
 =head1 backward compatibility
@@ -1015,6 +1022,12 @@ sub initTree {
                                 } elsif ($length eq 0) {
                                         $self->appendLastFolder(@$tree[$i], \@{@$tree[$i]->{subtree}});
                                 }
+                        } elsif (defined @$tree[$i]->{empty}) {
+                                if($length > 0) {
+                                        $self->appendEmptyFolder(@$tree[$i]);
+                                } elsif ($length eq 0) {
+                                        $self->appendLastEmptyFolder(@$tree[$i]);
+                                }
                         } else {
                                 if($length > 0) {
                                         $self->appendNode(@$tree[$i]);
@@ -1096,7 +1109,7 @@ sub appendFolder {
         my $tt;
 
         foreach my $key (keys %{$node}) {
-                $tt .= $key . '="' . $node->{$key} . '" ' if(exists $anker{$key});
+                $tt .= $key . '="' . $node->{$key} . '" ' if($anker{$key} && $node->{$key});
         }
         my $st = $node->{style} = (($columns > 0 or defined $node->{addition}) and not defined $node->{style}) ? 'style="white-space:nowrap;"' : '';
         my $addon =
@@ -1105,7 +1118,7 @@ sub appendFolder {
           : "&#160;<a $tt >$node->{text}</a>&#160;";
         my $minusnode = $clasic ? "clasicMinusNode$size" : "minusNode$size";
         $self->{tree} .=
-          qq(<tr><td  id="$id.node" class="$minusnode"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="spacer" onclick="$onclick"/></td><td align="left" class="$FolderClass" id="$id.folder"><table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendFolder" width="100%"><tr><td $st valign="top" width="$size"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="spacer" onclick="$onclick"/></td><td $st align="left">$addon</td></tr></table></td></tr><tr id="$id"><td class="submenuDeco$size"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" alt="spacer"/></td><td><table align="left" border="0" cellpadding="0" cellspacing="0"   summary="appendFolder" width="100%"><colgroup><col width="$size"/></colgroup>);
+          qq(<tr><td  id="$id.node" class="$minusnode"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" onclick="$onclick"/></td><td align="left" class="$FolderClass" id="$id.folder"><table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendFolder" width="100%"><tr><td $st valign="top" width="$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" onclick="$onclick"/></td><td $st align="left">$addon</td></tr></table></td></tr><tr id="$id"><td class="submenuDeco$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" alt=""/></td><td><table align="left" border="0" cellpadding="0" cellspacing="0"   summary="appendFolder" width="100%"><colgroup><col width="$size"/></colgroup>);
         if($columns > 0) {
                 my $class = $border ? "columnsFolderBorder$size" : "columnsFolder$size";
                 $self->{subtree} .= qq(<tr id="tr$id">);
@@ -1122,7 +1135,7 @@ sub appendFolder {
                 for(my $i = 0 ; $i < @$tmpref ; $i++) {
                         $self->{js}{$ty}[$i] = $$tmpref[$i];
                 }
-                @$tmpref = undef;
+                undef @$tmpref;
         }
         $self->{tree} .= '</table></td></tr>';
 }
@@ -1157,7 +1170,7 @@ sub appendLastFolder {
         my $tt;
 
         foreach my $key (keys %{$node}) {
-                $tt .= $key . '="' . $node->{$key} . '" ' if(exists $anker{$key});
+                $tt .= $key . '="' . $node->{$key} . '" ' if($anker{$key} && $node->{$key});
         }
         my $st = $node->{style} = (($columns > 0 or defined $node->{addition}) and not defined $node->{style}) ? 'style="white-space:nowrap;"' : '';
         my $addon =
@@ -1166,7 +1179,7 @@ sub appendLastFolder {
           : "&#160;<a $tt>$node->{text}</a>&#160;";
         my $lastminusnode = $clasic ? "clasicLastMinusNode$size" : "lastMinusNode$size";
         $self->{tree} .=
-          qq(<tr><td id="$id.node" class="$lastminusnode" onclick="$onclick"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="spacer" /></td><td align="left" class="$FolderClass" id="$id.folder"><table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendLastFolder" width="100%"><tr><td $st valign="top" width="$size"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" alt="spacer" onclick="$onclick"/></td><td $st align="left">$addon</td></tr></table></td></tr><tr id="$id"><td><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="spacer"/></td><td><table align="left" width="100%" cellpadding="0" cellspacing="0" border="0" summary="appendLastFolder"><colgroup><col width="$size"/></colgroup>);
+          qq(<tr><td id="$id.node" class="$lastminusnode" onclick="$onclick"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" /></td><td align="left" class="$FolderClass" id="$id.folder"><table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendLastFolder" width="100%"><tr><td $st valign="top" width="$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" alt="" onclick="$onclick"/></td><td $st align="left">$addon</td></tr></table></td></tr><tr id="$id"><td><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt=""/></td><td><table align="left" width="100%" cellpadding="0" cellspacing="0" border="0" summary="appendLastFolder"><colgroup><col width="$size"/></colgroup>);
         if($columns > 0) {
                 my $class = $border ? "columnsFolderBorder$size" : "columnsFolder$size";
                 $self->{subtree} .= qq(<tr id="tr$id">);
@@ -1183,11 +1196,117 @@ sub appendLastFolder {
                 for(my $i = 0 ; $i < @$tmpref ; $i++) {
                         $self->{js}{$ty}[$i] = $$tmpref[$i];
                 }
-                @$tmpref = undef;
+                undef @$tmpref;
         }
         $self->{tree} .= '</table></td></tr>';
 }
 
+=head2 appendEmptyFolder
+
+called by initTree(), append a empty Folder.
+
+=cut
+
+sub appendEmptyFolder {
+        my $self = shift;
+        my $node = shift;
+        ++$id;
+        my ($tmpref, $ty);
+        if($columns > 0) {
+                $ty = $id;
+                foreach my $key (keys %openArrays) {
+                        push @{$openArrays{$key}}, $id;
+                }
+                $tmpref = \@{$openArrays{$id}};
+        }
+        my $onclick = qq(location.href='$node->{href}');
+        $node->{class} = defined $node->{class} ? $node->{class} : "treeviewLink$size";
+        my $FolderClass = defined $node->{folderclass} ? $node->{folderclass} . "Closed$size" : "folderClosed$size";
+        $node->{title} = defined $node->{title} ? $node->{title} : $node->{text};
+        my $tt;
+        foreach my $key (keys %{$node}) {
+                $tt .= $key . '="' . $node->{$key} . '" ' if($anker{$key} && $node->{$key});
+        }
+        my $st = $node->{style} = (($columns > 0 or defined $node->{addition}) and not defined $node->{style}) ? 'style="white-space:nowrap;"' : '';
+        my $addon =
+          defined $node->{addition}
+          ? qq(<table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendEmptyFolder" width="100%"><tr><td $st>&#160;<a $tt>$node->{text}</a>&#160;</td><td $st>$node->{addition}</td></tr></table>)
+          : "&#160;<a $tt >$node->{text}</a>&#160;";
+        my $plusnode = $clasic ? "clasicPlusNode$size" : "plusNode$size";
+        $self->{tree} .=
+          qq(<tr><td  id="$id.node" class="$plusnode"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" onclick="$onclick"/></td><td align="left" class="$FolderClass" id="$id.folder"><table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendFolder" width="100%"><tr><td $st valign="top" width="$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" onclick="$onclick"/></td><td $st align="left">$addon</td></tr></table></td></tr>);
+        if($columns > 0) {
+                my $class = $border ? "columnsFolderBorder$size" : "columnsFolder$size";
+                $self->{subtree} .= qq(<tr id="tr$id">);
+                for(my $i = 0 ; $i < $columns ; $i++) {
+                        if(defined $node->{columns}[$i]) {
+                                my $txt = $node->{columns}[$i];
+                                $self->{subtree} .= qq(<td class="$class">$txt</td>);
+                        }
+                }
+                $self->{subtree} .= '</tr>';
+                for(my $i = 0 ; $i < @$tmpref ; $i++) {
+                        $self->{js}{$ty}[$i] = $$tmpref[$i];
+                }
+                undef @$tmpref;
+        }
+}
+
+=head2 appendLastEmptyFolder
+
+$self->appendLastEmptyFolder($node);
+
+called by initTree() if the last item of the (sub)Tree is a folder.
+
+=cut
+
+sub appendLastEmptyFolder {
+        my $self = shift;
+        my $node = shift;
+        $id++;
+        my $tmpref;
+        my $ty;
+        if($columns > 0) {
+                $ty = $id;
+                foreach my $key (keys %openArrays) {
+                        push @{$openArrays{$key}}, $id;
+                }
+                $tmpref = \@{$openArrays{$id}};
+        }
+        my $onclick = qq(location.href='$node->{href}');
+        $node->{class} = defined $node->{class} ? $node->{class} : "treeviewLink$size";
+        my $FolderClass = defined $node->{folderclass} ? $node->{folderclass} . "Closed$size" : "folderClosed$size";
+        $node->{title} = defined $node->{title} ? $node->{title} : $node->{text};
+        my $tt;
+
+        foreach my $key (keys %{$node}) {
+                $tt .= $key . '="' . $node->{$key} . '" ' if($anker{$key} && $node->{$key});
+        }
+        my $st = $node->{style} = (($columns > 0 or defined $node->{addition}) and not defined $node->{style}) ? 'style="white-space:nowrap;"' : '';
+        my $addon =
+          defined $node->{addition}
+          ? qq(<table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendLastFolder"  width="100%"><tr><td $st>&#160;<a $tt >$node->{text}</a>&#160;</td><td $st>$node->{addition}</td></tr></table>)
+          : "&#160;<a $tt>$node->{text}</a>&#160;";
+        my $lastpusnode = $clasic ? "clasicLastPlusNode$size" : "lastPlusNode$size";
+        $self->{tree} .=
+          qq(<tr><td id="$id.node" class="$lastpusnode" onclick="$onclick"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" /></td><td align="left" class="$FolderClass" id="$id.folder"><table align="left" border="0" cellpadding="0" cellspacing="0" summary="appendLastFolder" width="100%"><tr><td $st valign="top" width="$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" alt="" onclick="$onclick"/></td><td $st align="left">$addon</td></tr></table></td></tr>);
+        if($columns > 0) {
+                my $class = $border ? "columnsFolderBorder$size" : "columnsFolder$size";
+                $self->{subtree} .= qq(<tr id="tr$id">);
+                for(my $i = 0 ; $i < $columns ; $i++) {
+                        if(defined $node->{columns}[$i]) {
+                                my $txt = $node->{columns}[$i];
+                                $self->{subtree} .= qq(<td class="$class">$txt</td>);
+                        }
+                }
+                $self->{subtree} .= '</tr>';
+                for(my $i = 0 ; $i < @$tmpref ; $i++) {
+                        $self->{js}{$ty}[$i] = $$tmpref[$i];
+                }
+                undef @$tmpref;
+        }
+}
+
 =head2 appendNode
 
 $self->appendLastNode(\$node);
@@ -1210,7 +1329,7 @@ sub appendNode {
         }
         my $tt;
         foreach my $key (keys %{$node}) {
-                $tt .= $key . '="' . $node->{$key} . '" ' if(exists $anker{$key});
+                $tt .= $key . '="' . $node->{$key} . '" ' if($anker{$key} && $node->{$key});
         }
         my $st = $node->{style} = (($columns > 0 or defined $node->{addition}) and not defined $node->{style}) ? 'style="white-space:nowrap;"' : '';
         my $addon =
@@ -1219,7 +1338,7 @@ sub appendNode {
           : "&#160;<a $tt>$node->{text}</a>";
         my $paddingLeft = $size+ 2 . "px";
         $self->{tree} .=
-          qq(<tr><td class="node$size"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="spacer" align="middle"/></td><td align="left"  style="background-image:url('$pref/style/$style/$size/mimetypes/$node->{image}');background-repeat:no-repeat;cursor:pointer;padding-left:$paddingLeft">$addon</td></tr>);
+          qq(<tr><td class="node$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="" align="middle"/></td><td align="left"  style="background-image:url('$prefix/style/$style/$size/mimetypes/$node->{image}');background-repeat:no-repeat;cursor:pointer;padding-left:$paddingLeft">$addon</td></tr>);
         if($columns > 0) {
                 $self->{subtree} .= qq(<tr id="tr$id">);
                 my $class = $border ? "columnsNodeBorder$size" : "columnsNode$size";
@@ -1255,7 +1374,7 @@ sub appendLastNode {
         }
         my $tt;
         foreach my $key (keys %{$node}) {
-                $tt .= $key . '="' . $node->{$key} . '" ' if(exists $anker{$key});
+                $tt .= $key . '="' . $node->{$key} . '" ' if($anker{$key} && $node->{$key});
         }
         my $st = $node->{style} = (($columns > 0 or defined $node->{addition}) and not defined $node->{style}) ? 'style="white-space:nowrap;"' : '';
         my $addon =
@@ -1264,7 +1383,7 @@ sub appendLastNode {
           : "&#160;<a $tt>$node->{text}</a>";
         my $paddingLeft = $size+ 2 . "px";
         $self->{tree} .=
-          qq(<tr><td class="lastNode$size"><img src="$pref/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt="spacer"/></td><td align="left"  style="background-image:url('$pref/style/$style/$size/mimetypes/$node->{image}');background-repeat:no-repeat;cursor:pointer;padding-left:$paddingLeft;">$addon</td></tr>);
+          qq(<tr><td class="lastNode$size"><img src="$prefix/style/$style/$size/html-menu-treeview/spacer.gif" border="0" width="$size" height="$size" alt=""/></td><td align="left"  style="background-image:url('$prefix/style/$style/$size/mimetypes/$node->{image}');background-repeat:no-repeat;cursor:pointer;padding-left:$paddingLeft;">$addon</td></tr>);
         if($columns > 0) {
                 my $class = $border ? "columnsLastNodeBorder$size" : "columnsLastNode$size";
                 $self->{subtree} .= qq(<tr id="tr$id">);
